/*
 * CRRCsim - the Charles River Radio Control Club Flight Simulator Project
 *
 *  Copyright (C) 2010 Joel Lienard (original author)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
  

/*
 * crrc_viewwind.cpp
 * classe  CGUIViewWindDialog
 * visualisation of slope and wind profils, for debug purpose
 */
 
#if (viewwind==1)
#include "crrc_gui_viewwind.h"

#include <stdio.h>
#include <stdlib.h>

#include "util.h"
#include "../crrc_main.h"
#include "../global.h"
#include "../mod_misc/lib_conversions.h"
#include "../mod_misc/SimpleXMLTransfer.h"
#include "../mod_windfield/windfield.h"
#include "../mod_landscape/crrc_scenery.h"

static void CGUIViewWindDialogCallback(puObject *obj);
static void CGUIViewWindSliderCallback(puObject *obj);
static void  draw_wind(class puObject *, int dx, int dy, void *);


#define BUTTON_BOX_HEIGHT   (2*DLG_DEF_SPACE+DLG_DEF_BUTTON_HEIGHT)
#define SLIDER_W 330
#define SLIDER_H 18
#define COMBO_H  20
#define COMBO_W  230
#define LABEL_W  150
#define NUM_W    80
#define GRAPHE_W    900
#define GRAPHE_H    450
#define SLIDER_FRACTION 0.05

CGUIViewWindDialog::CGUIViewWindDialog()
  : CRRCDialog()
{
 
  // Create widgets
  //
  
  
//inital values
position = 0;
direction = cfg->wind->getDirection();
distance_max= 500;
distance_min= -10;
height_sky = 100;


int ypos = BUTTON_BOX_HEIGHT;
int xpos2 = LABEL_W + DLG_DEF_SPACE + SLIDER_W;

//slider direction
  slider_direction = new crrcSlider(LABEL_W + DLG_DEF_SPACE, ypos,
                                    LABEL_W + DLG_DEF_SPACE + SLIDER_W,  ypos + SLIDER_H,
                                    NUM_W);
  //ypos += (SLIDER_H + DLG_DEF_SPACE);
  slider_direction->setLabelPlace(PUPLACE_CENTERED_LEFT);
  slider_direction->setLabel("direction(degrees)");
  slider_direction->setSliderFraction(SLIDER_FRACTION);
  slider_direction->setMinValue(0);
  slider_direction->setMaxValue(360);
  slider_direction->setStepSize(1);
  slider_direction->setValue(direction);
  slider_direction->setCallback(CGUIViewWindSliderCallback);
  slider_direction->setUserData(this);

//slider position
  slider_position = new crrcSlider(xpos2 + LABEL_W + DLG_DEF_SPACE, ypos,
                                   xpos2 + LABEL_W + DLG_DEF_SPACE + SLIDER_W,  ypos + SLIDER_H,
                                    NUM_W);
  ypos += (SLIDER_H + DLG_DEF_SPACE);
  slider_position->setLabelPlace(PUPLACE_CENTERED_LEFT);
  slider_position->setLabel("position (left<->right");
  slider_position->setSliderFraction(SLIDER_FRACTION);
  slider_position->setMinValue(-1000);
  slider_position->setMaxValue(+1000);
  slider_position->setStepSize(1);
  slider_position->setValue(position);
  slider_position->setCallback(CGUIViewWindSliderCallback);
  slider_position->setUserData(this);
  
  //slider max distance
  slider_dmax = new crrcSlider(LABEL_W + DLG_DEF_SPACE, ypos,
                                    LABEL_W + DLG_DEF_SPACE + SLIDER_W,  ypos + SLIDER_H,
                                    NUM_W);
  //ypos += (SLIDER_H + DLG_DEF_SPACE);
  slider_dmax->setLabelPlace(PUPLACE_CENTERED_LEFT);
  slider_dmax->setLabel("distance max");
  slider_dmax->setSliderFraction(SLIDER_FRACTION);
  slider_dmax->setMinValue(0);
  slider_dmax->setMaxValue(+3000);
  slider_dmax->setStepSize(10);
  slider_dmax->setValue(distance_max);
  slider_dmax->setCallback(CGUIViewWindSliderCallback);
  slider_dmax->setUserData(this);

  //slider min distance
  slider_dmin = new crrcSlider(xpos2 + LABEL_W + DLG_DEF_SPACE, ypos,
                               xpos2 + LABEL_W + DLG_DEF_SPACE + SLIDER_W,  ypos + SLIDER_H,
                                    NUM_W);
  ypos += (SLIDER_H + DLG_DEF_SPACE);
  slider_dmin->setLabelPlace(PUPLACE_CENTERED_LEFT);
  slider_dmin->setLabel("distance min");
  slider_dmin->setSliderFraction(SLIDER_FRACTION);
  slider_dmin->setMinValue(-200);
  slider_dmin->setMaxValue(+0);
  slider_dmin->setStepSize(10);
  slider_dmin->setValue(distance_min);
  slider_dmin->setCallback(CGUIViewWindSliderCallback);
  slider_dmin->setUserData(this);

  //slider height of sky
  slider_hsky = new crrcSlider( LABEL_W + DLG_DEF_SPACE, ypos,
                                LABEL_W + DLG_DEF_SPACE + SLIDER_W,  ypos + SLIDER_H,
                                    NUM_W);
  ypos += (SLIDER_H + DLG_DEF_SPACE);
  slider_hsky->setLabelPlace(PUPLACE_CENTERED_LEFT);
  slider_hsky->setLabel("sky height");
  slider_hsky->setSliderFraction(SLIDER_FRACTION);
  slider_hsky->setMinValue(-10);
  slider_hsky->setMaxValue(400);
  slider_hsky->setStepSize(10);
  slider_hsky->setValue(height_sky);
  slider_hsky->setCallback(CGUIViewWindSliderCallback);
  slider_hsky->setUserData(this);

  // graphical frame
  graphe = new puFrame ( DLG_DEF_SPACE,         ypos, 
                      DLG_DEF_SPACE + GRAPHE_W,   ypos + GRAPHE_H);
  ypos += (GRAPHE_H + DLG_DEF_SPACE);
  graphe->setRenderCallback ( draw_wind, this );

  close();
  //size
  int sizew = GRAPHE_W + DLG_DEF_SPACE;
  if(sizew < 2*xpos2) sizew = 2*xpos2 ;
  setSize(sizew + DLG_DEF_SPACE, ypos);
  setCallback(CGUIViewWindDialogCallback);


  // center the dialog on screen
  int wwidth, wheight;
  int current_width = getABox()->max[0] - getABox()->min[0];
  int current_height = getABox()->max[1] - getABox()->min[1];
  puGetWindowSize(&wwidth, &wheight);
  setPosition(wwidth/2 - current_width/2, wheight/2 - current_height/2);

  reveal();
}

/**
 * Destroy the dialog.
 */
CGUIViewWindDialog::~CGUIViewWindDialog()
{
}


/** The dialog's callback.
 *
 */
void CGUIViewWindDialogCallback(puObject *obj)
{
  if (obj->getIntegerValue() == CRRC_DIALOG_OK)
  {
    // Dialog left by clicking OK
    //CGUIViewWindDialog* dlg   = (CGUIViewWindDialog*)obj;
    //nothing...
   }  
  puDeleteObject(obj);
}

//slider callback
void CGUIViewWindSliderCallback(puObject *obj)
{
 CGUIViewWindDialog* dlg = (CGUIViewWindDialog*)(obj->getUserData());
 dlg->direction = dlg->slider_direction->getIntegerValue();
 dlg->position =  dlg->slider_position->getIntegerValue();
 dlg->distance_min =  dlg->slider_dmin->getIntegerValue();
 dlg->distance_max =  dlg->slider_dmax->getIntegerValue();
 dlg->height_sky =  dlg->slider_hsky->getIntegerValue();
 }


void  draw_wind(puObject *obj, int dx, int dy,void *data)
{
  CGUIViewWindDialog* dlg = (CGUIViewWindDialog*)data;
  dlg->draww(obj,dx,dy);
  }

void  CGUIViewWindDialog::draww(puObject *obj, int dx, int dy)
{  
float  wind_ref = cfg->wind->getVelocity();//nominal wind
  //get draw rectangle
  int x,y,w,h;
  int i;
  const int marge=20;
  obj->getPosition ( &x, &y );
  obj->getSize ( &w, &h );
  x +=dx; y +=dy;
   x += marge;y += marge;
   w -= (2*marge);  h -= (2*marge);
   float h0=(float)h/(float)w;
  //clip
  glScissor(x,y,w,h);
  glEnable(GL_SCISSOR_TEST);
  
  //draw
  glPushMatrix();
  glLoadIdentity ();
  glTranslatef(x,y,0);
 
  int n=50;//nb of points per trace
  int nc=15;//nb of trace
  float Dmax = distance_max;
  float Dmin = distance_min;
  float angle = direction/180.0*M_PI;
  float cosi = cos(angle);
  float sinu = sin(angle);
  float ddx = Dmax/n;
  glScalef(w,w,1);
     
  float zmax= -9999;
  float zmin= 9999;
  float orig_x = position*sinu;
  float orig_y = position*cosi;
  float x1;
  float height[n];

  // get slope profil and calcul min and max
  for(i=0;i<n;i++)
    {
    float d = Dmin+ddx*i;
    float z= Global::scenery->getHeight(orig_x + d*cosi,orig_y + d*sinu);
    //z  positiv_up
    if( z > zmax) zmax = z;
    if( z < zmin) zmin = z;
    height[i] = z;
    }
  //draw slope profil
  glColor3f (0.1, 0.1, 0.1); 
  glBegin(GL_POLYGON);
  glVertex2f(0.,0.);
  for(i=0;i<n;i++)
    {
    float d = Dmin+ddx*i;
    float z =height[i];
    x1 = (d-Dmin)/Dmax;
    float y1= (z - zmax - height_sky)/Dmax + h0;// max on top
    glVertex2f(x1,y1);
    }
   glVertex2f(x1,0.);
   glEnd();  
 //draw wind traces
  float TRAIT=2./n; //nominal length of wind markers
  glColor3f (0.0, 0.0, 0.9); 
  glBegin(GL_LINES);
  float DH=30;
  int j;
  for(j=1;j<=nc;j++)
  for(i=0;i<n;i++)
    {
    float d = Dmin+ddx*i;
    float z =height[i] + DH*j;
    float x_wind, y_wind, z_wind;
    Global::scenery->getWindComponents(orig_x + d*cosi, orig_y + d*sinu, -z,/*z negativ=up*/
                              &x_wind, &y_wind, &z_wind);
    //float h_wind; //horizontal wind
    float d_wind; //wind on direction analysis
    //h_wind = sqrt(x_wind*x_wind + y_wind*y_wind);//horirontal wind
    d_wind =  cosi*x_wind + sinu*y_wind;//wind component en direction analysis
    x1 = (d-Dmin)/Dmax;
    float y1= (z - zmax - height_sky)/Dmax + h0;// max on top
    arrow(x1,y1,d_wind/wind_ref,-z_wind/wind_ref,TRAIT); 
    }
  glEnd();  
  glDisable(GL_SCISSOR_TEST);
  glPopMatrix();

}
void CGUIViewWindDialog::arrow(float x,float y,float dx,float dy,float lg)
  {
  dx *= lg;
  dy *= lg;
  float dx2=dx/10;
  float dy2=dy/10;
  glVertex2f(x-dy2,y+dx2);
  glVertex2f(x +dx ,y + dy);
  glVertex2f(x+dy2,y-dx2);
  glVertex2f(x +dx ,y + dy);
  }
#endif // (viewwind==1)

